@@ -6,6 +6,7 @@ import json
 import logging
 import secrets
 import struct
+import sys
 import uuid
 import plotly
 import dash
@@ -127,12 +128,17 @@ class DependencyCollection:
             if i == len(multi_index) - 1:
                 e[j] = value
 
-    def append(self, value, flex_key=None):
+    def append(self, value, flex_key=None, index=None):
         i = len(self._index)
         if isinstance(self.structure, list):
-            self.structure.append(value)
-            self._re_index()
-            return i
+            if index is not None:
+                self.structure.insert(index, value)
+                self._re_index()
+                return index
+            else:
+                self.structure.append(value)
+                self._re_index()
+                return i
         if isinstance(self.structure, dict):
             flex_key = f"{DEPENDENCY_APPEND_PREFIX}{i}" if flex_key is None else flex_key
             self.structure[flex_key] = value
@@ -163,6 +169,15 @@ def collect_args(args: Union[Tuple[Any], List[Any]], inputs, outputs):
     return DependencyCollection(inputs), DependencyCollection(outputs)
 
 
+class DummyDependency(DashDependency):
+    """
+    Object used to represent dummy dependencies.
+    """
+
+    def __init__(self, component_id, component_property):
+        super().__init__(component_id, component_property)
+
+
 class CallbackBlueprint:
     def __init__(self, *args, **kwargs):
         # Collect args "normally".
@@ -174,6 +189,10 @@ class CallbackBlueprint:
             self.inputs = DependencyCollection(kwargs.pop("inputs"), keyword="inputs")
         if "state" in kwargs:
             raise ValueError("Please use the 'inputs' keyword instead of the 'state' keyword.")
+        # Collect dummy elements.
+        if kwargs.get("background", False) and "progress" in kwargs:
+            # This element represents the set_progress function.
+            self.inputs.append(DummyDependency("function", "set_progress"), index=0, flex_key="set_progress")
         # Collect the rest.
         self.kwargs: Dict[str, Any] = kwargs
         self.f = None
@@ -185,6 +204,10 @@ class CallbackBlueprint:
             s = dep_col.structure
             if isinstance(s, list) and len(s) == 1:
                 s = s[0]
+            if isinstance(s, DummyDependency):
+                continue
+            if isinstance(s, list):
+                s = [dep for dep in s if not isinstance(dep, DummyDependency)]
             if dep_col.keyword is None:
                 dep_args.append(s)
             else:
@@ -326,9 +349,19 @@ class DashProxy(dash.Dash):
     def clientside_callback(self, clientside_function, *args, **kwargs):
         return self.blueprint.clientside_callback(clientside_function, *args, **kwargs)
 
-    def _setup_server(self):
-        # Register the callbacks.
+    def long_callback(self, *_args, **_kwargs):
+        raise NotImplementedError(
+            "The 'long_callback(..)' syntax is not supported, please use 'callback(background=True, ...)' instead.")
+
+    def register_celery_tasks(self):
+        if sys.argv[0].endswith("celery"):
+            self.register_callbacks()
+
+    def register_callbacks(self):
         self.blueprint.register_callbacks(super())
+
+    def _setup_server(self):
+        self.register_callbacks()
         # Proceed as normally.
         super()._setup_server()
         # Remap callback bindings to enable callback registration via the 'before_first_request' hook.
@@ -1137,8 +1170,8 @@ def _pack_outputs(callback):
                 update_needed = False
                 for i, output in enumerate(callback.outputs):
                     # Filter out Triggers (a little ugly to do here, should ideally be handled elsewhere).
-                    is_trigger = [isinstance(item, Trigger) for item in callback.inputs]
-                    filtered_args = [arg for i, arg in enumerate(args) if not is_trigger[i]]
+                    to_skip = [isinstance(item, Trigger) for item in callback.inputs]
+                    filtered_args = [arg for i, arg in enumerate(args) if not to_skip[i]]
                     # Generate unique ID.
                     unique_id = _get_cache_id(f, output, list(filtered_args), output.session_check, output.arg_check)
                     unique_ids.append(unique_id)
@@ -1167,8 +1200,8 @@ def _pack_outputs(callback):
                 serverside_output = isinstance(callback.outputs[i], ServersideOutput)
                 if serverside_output or memoize:
                     # Filter out Triggers (a little ugly to do here, should ideally be handled elsewhere).
-                    is_trigger = [isinstance(item, Trigger) for item in callback.inputs]
-                    filtered_args = [arg for i, arg in enumerate(args) if not is_trigger[i]]
+                    to_skip = [isinstance(item, (Trigger, DummyDependency)) for item in callback.inputs]
+                    filtered_args = [arg for i, arg in enumerate(args) if not to_skip[i]]
                     unique_id = _get_cache_id(f, output, list(filtered_args), output.session_check, output.arg_check)
                     output.backend.set(unique_id, data[i])
                     # Replace only for server side outputs.
@@ -1187,7 +1220,6 @@ def _get_cache_id(func, output, args, session_check=None, arg_check=True):
         all_args += list(args)
     if session_check:
         all_args += [_get_session_id()]
-    print(all_args)
     return hashlib.md5(json.dumps(all_args).encode()).hexdigest()
 
 
